@@ -126,12 +126,14 @@ pub fn write_header<T: std::fmt::Write>(
 /// Create a simple theme.
 ///
 pub trait SimpleTheme {
-    fn simple_theme<T: fmt::Write>(&mut self, a: T) -> std::fmt::Result;
-    fn simple_theme_dark<T: fmt::Write>(&mut self, a: T) -> std::fmt::Result;
+    fn simple_theme<T: fmt::Write>(self, a: T) -> std::fmt::Result;
+    fn simple_theme_dark<T: fmt::Write>(self, a: T) -> std::fmt::Result;
 }
 
-impl<D: Disp> SimpleTheme for Plotter<D> {
-    fn simple_theme<T: fmt::Write>(&mut self, mut a: T) -> std::fmt::Result {
+impl<P: PlotIterator<Item = (B::X, B::Y)>, K: Borrow<Canvas>, B: BaseFmt> SimpleTheme
+    for Plotter<P, K, B>
+{
+    fn simple_theme<T: fmt::Write>(self, mut a: T) -> std::fmt::Result {
         let dim = self.get_dim();
 
         write!(
@@ -144,7 +146,7 @@ impl<D: Disp> SimpleTheme for Plotter<D> {
         )
     }
 
-    fn simple_theme_dark<T: fmt::Write>(&mut self, mut a: T) -> std::fmt::Result {
+    fn simple_theme_dark<T: fmt::Write>(self, mut a: T) -> std::fmt::Result {
         let dim = self.get_dim();
 
         write!(
